/**
 * EasyBeans
 * Copyright (C) 2008 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClientCluster.java 2707 2008-03-25 17:51:28Z loris $
 * --------------------------------------------------------------------------
 */

package org.ow2.easybeans.examples.cluster;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Hashtable;
import java.util.Properties;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.ow2.cmi.rpc.CMIProxy;

/**
 * Simple client of the stateless which clustering infos are built from
 * deployment descriptor.
 * @author eyindanga
 */
public final class ClientClusterDD {

    static long pid = System.currentTimeMillis() % 1000;

    /**
     * Utility class.
     */
    private ClientClusterDD() {

    }

    /**
     * Main method.
     * @param args the arguments (not required)
     * @throws Exception if exception is found.
     */
    public static void main(final String[] args) throws Exception {

        int i = 0;
        String msg = null;
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyMMddHHmmss");
        Properties prop = new Properties();
        prop.put(ClusterProperty.CMI_INITIAL_CONTEXT_FACTORY_KEY.getProperty(), ClusterProperty.DEFAULT_INITIAL_CONTEXT_FACTORY.getProperty());
        File temp = File.createTempFile(ClusterProperty.DEFAULT_CMI_FILE_EXT.getProperty(),
                ClusterProperty.DEFAULT_CMI_FILE_EXT.getProperty());
        // Delete temp file when program exits.
        temp.deleteOnExit();
        System.setProperty(ClusterProperty.CMI_CONF_URL_KEY.getProperty(), temp.getAbsolutePath());
        Context initialContext = getInitialContext();
        boolean clusteringEnabled = false;
        ClusterRemote statelessBean = (ClusterRemote) initialContext.lookup("clusterBeanConfiguredByXML");

        if (statelessBean instanceof CMIProxy) {
            clusteringEnabled = true;
            System.out.println("Clustering is enabled.");
        }

        while (i < Utils.LOOP_NUMBER) {
            i++;
            msg = "Request[T=" + pid + ", HD=" + dateFormat.format(new Date()) + ", S=" + i + "]";
            if (clusteringEnabled) {
                System.out.println("The stub used for the next request is:\n"
                        + ((CMIProxy) statelessBean).getCurrentCMIRef_CMI());
            }
            // invoke the EJB
            String ezbServerDescription = statelessBean.getEZBServerDescription(msg);

            System.out.println(msg + " -> " + ezbServerDescription);

            Thread.sleep(Utils.PAUSE_TIME);
        }

    }

    /**
     * @return Returns the InitialContext.
     * @throws NamingException If the Context cannot be created.
     */
    private static Context getInitialContext() throws NamingException {

        // if user don't use jclient/client container
        // we can specify the InitialContextFactory to use
        // But this is *not recommended*.
        Hashtable<String, Object> env = new Hashtable<String, Object>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, getInitialContextFactory());

        // Usually a simple new InitialContext() without any parameters is
        // sufficent.
        // return new InitialContext();

        return new InitialContext(env);
    }

    /**
     * Returns a configurable InitialContextFactory classname.<br/> Can be
     * configured with the <code>easybeans.client.initial-context-factory</code>
     * System property.
     * @return Returns a configurable InitialContextFactory classname.
     */
    private static String getInitialContextFactory() {
        String prop = System.getProperty(ClusterProperty.DEFAULT_EZB_INITIAL_CONTEXT_FACTORY.getProperty());
        // If not found, use the default
        if (prop == null) {
            prop = ClusterProperty.DEFAULT_INITIAL_CONTEXT_FACTORY.getProperty();
        }
        return prop;
    }

}
